@@ -5,7 +5,7 @@ oer+MySQL - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2001 EQU <equ@equnet.org>
+Copyright (C) 2000-2002 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -115,13 +115,16 @@ extern int deltaflags(struct channel *, char *, char);
 extern int isvalidchannel(char *);
 extern int hasuserflags(struct channel *, char *, char *, char *);
 extern int noexpiredlogons(struct channel *);
+extern int delchannelban(struct channel *, char *);
 extern unsigned int getrandom(unsigned int);
-extern struct authed *addnewauthed(struct channel *, time_t, struct botuser *, char *);
+extern struct authed *addnewauthed(struct channel *, time_t, char *, char *);
 extern struct botuser *addbotuser(char *, char *);
 extern struct botuser *addnewadmin(char *, char *);
 extern struct botuser *addnewuser(struct channel *, char *, char *);
+extern struct botuser *usercopy(struct channel *, char *, struct channel *, char *);
 extern struct channel *addnewchannel(char *);
 extern struct channel *getchptr(char *);
+extern struct channelban *addnewchannelban(struct channel *, char *, char *, time_t);
 extern struct chanuser *userjoined(struct channel *, char *, char *, int, int, int, int);
 extern struct chanuser *getcuptr(struct channel *, char *);
 extern struct state *emptystate(void);
@@ -152,6 +155,7 @@ extern time_t lastoff(struct channel *, char *, char *);
 extern void sendadverts(struct channel *, char *, char *);
 extern void resetparts(struct channel *, char *, char *);
 extern void initpubmsgs(struct channel *);
+extern void initchannelbans(struct channel *);
 extern void freenickchanges(struct channel *, char *);
 extern void freepubmsguser(struct channel *, char *, char *);
 extern void lockchan(struct channel *, char *, int, char *, char *);
@@ -209,3 +213,5 @@ extern void cleanautheds(struct channel *, int);
 extern void listchanusers(void);
 extern void timed_del(struct timed *);
 extern void mmode_del(struct channel *, struct mmode *);
+extern void validateflags(char *, char *, int);
+extern void resetfloodcounters(struct channel *, char *);
