@@ -5,7 +5,7 @@ oer+MySQL - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2001 EQU <equ@equnet.org>
+Copyright (C) 2000-2002 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -163,6 +163,8 @@ enum WHICHCOMMAND_RETURN_VALUES {
         OER_WHICHCOMMAND_CLONECHANNEL,
         OER_WHICHCOMMAND_SSTATS,
         OER_WHICHCOMMAND_DATE,
+        OER_WHICHCOMMAND_USERCOPY,
+        OER_WHICHCOMMAND_CLEARBANS,
 	OER_WHICHCOMMAND_INVALID
 };
 
@@ -244,6 +246,18 @@ enum OER_TIMED_PRIORITIES {
         OER_TIMED_PRIORITY_CHANNEL_INVALID
 };
 
+enum OER_FLAGS_TYPE {
+        OER_FLAGS_TYPE_GLOBAL,
+        OER_FLAGS_TYPE_CHANNEL,
+        OER_FLAGS_TYPE_ADMIN,
+        OER_FLAGS_TYPE_USER
+};
+
+#define OER_FLAGS_GLOBAL "dfglmnpqs"
+#define OER_FLAGS_CHANNEL "!aAbcDefFGklLmMnNoOpPqrsSuUtTvVwx"
+#define OER_FLAGS_ADMIN "dn"
+#define OER_FLAGS_USER "!adfmovrx"
+
 #define numofparams(x) (wordcount(x) - 1)
 
 /* from glib.h */
@@ -336,13 +350,14 @@ enum OER_TIMED_PRIORITIES {
 #define OER_MAX_LIST_LINES 5
 #define OER_HELP "For a complete command reference go to http://oer.equnet.org/help.html"
 #define OER_SERVER_IS_STONED "server stoned"
+#define OER_INTERNAL_DYNAMIC_ADMIN "*INTDYN*"
 #define DEFAULT_NICKBK_MESSAGE "prohibited nick"
 #define DEFAULT_WORDBK_MESSAGE "word ban-kick triggered, have a nice life..."
 #define DBSTATUS_MSG "At least one of the executed database queries failed, more \
 details possibly available on the oer+MySQL console"
 #define OER_REALNAME "http://oer.equnet.org/help.html"
 #define OER_UNKNOWN "unknown"
-#define OER_COPYRIGHT1 "oer+MySQL version 1.0-29, Copyright (C) 2000-2001 EQU <equ@equnet.org>"
+#define OER_COPYRIGHT1 "oer+MySQL version 1.0-30, Copyright (C) 2000-2002 EQU <equ@equnet.org>"
 #define OER_COPYRIGHT2 "oer+MySQL comes with ABSOLUTELY NO WARRANTY; for details see the\n\
 GNU General Public License (COPYING) provided with this distribution.\n\
 This is free software, and you are welcome to redistribute it under\n\
@@ -403,6 +418,9 @@ struct chanuser {
 	int voice;
 	int tobek;
 	int friend;
+	time_t isflood_last;
+        int isflood_lines;
+        int isflood_chars;
 	struct chanuser *prev;
 	struct chanuser *next;
 };
@@ -418,8 +436,8 @@ struct botuser {
 
 struct authed {
 	time_t at;
-	struct botuser *user;
 	char *userhost;
+	char *handle;
 	struct authed *prev;
 	struct authed *next;
 };
@@ -503,6 +521,14 @@ struct last_action {
         unsigned int barrier;
 };
 
+struct channelban {
+        char *ban;
+        char *setby;
+        time_t at;
+        struct channelban *next;
+        struct channelban *prev;
+};
+
 struct server {
 	char serverhost[HOSTLEN + 1];
 	int serverport;
@@ -558,6 +584,7 @@ struct channel {
 	struct authed *autheds;
 	struct advert *adverts;
 	struct mmode *mmodes;
+	struct channelban *channelbans;
 	struct channel *prev;
 	struct channel *next;
 };
