@@ -5,7 +5,7 @@ oer+MySQL - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2001 EQU <equ@equnet.org>
+Copyright (C) 2000-2002 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -87,11 +87,13 @@ void parseirc(char *serveroutput)
 	restptr = serveroutput + ppos;
 	/* the rest of the line depends on the command/numeric */
 	if(!strcasecmp(command, "001")) {
-		mystate->current_server->registered.done = 1;
-		/* we got a nick that the server accepted */
-		mystate->newnick = 0;
-		return;
-	}
+		/* we have registered & are connected to the server */
+                mystate->current_server->registered.done = 1;
+                /* force a userhost on our nick to find out our hostname */
+                snprintf(timed_str, WRITE_BUFFER_LENGTH, "USERHOST %s", mystate->nick);
+                timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, timed_str);
+                return;
+        }
 	if(!strcasecmp(command, "002")) {
 		return;
 	}
@@ -138,38 +140,69 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "302")) {
 		/* first char is ':' */
-		restptr++;
-		restpos = 0;
-		while(1) {
-			nrestpos = parse(restptr, restpos, " ", outstring, STRINGLEN, 0);
-			if(nrestpos == restpos || nrestpos < 0) {
-				break;
-			}
-			restpos = nrestpos;
-			memset(nick, 0, NICKLEN + 1);
-			tpos = 0;
-			if((tpos = parse(outstring, tpos, "=", nick, NICKLEN, 1)) < 0) {
-				return;
-			}
-			if(!strlen(nick)) {
-				/* USERHOST returns emptyline for invalid nicks */
-				return;
-			}
+                restptr++;
+                restpos = 0;
+                while(1) {
+                        nrestpos = parse(restptr, restpos, " ", outstring, STRINGLEN, 0);
+                        if(nrestpos == restpos || nrestpos < 0) {
+                                break;
+                        }
+                        restpos = nrestpos;
+                        memset(nick, 0, NICKLEN + 1);
+                        tpos = 0;
+                        if((tpos = parse(outstring, tpos, "=", nick, NICKLEN, 1)) < 0) {
+                                return;
+                        }
+                        if(!strlen(nick)) {
+                                /* USERHOST returns emptyline for invalid nicks */
+                                return;
+                        }
 			/* check for ircop, remove trailing star */
-			if(nick[strlen(nick) - 1] == '*') {
-				nick[strlen(nick) - 1] = '\0';
-				/* isoper */
-				for(this = mystate->channels; this != NULL; this = this->next) {
-					if(isonchan(this, nick)) {
-						changeuser(this, nick, 1, -1, -1, -1);
-					}
-				}
-			}
-			/* userhost is at outstring + tpos + 1 (first char is '+' or '-') */
+                        if(nick[strlen(nick) - 1] == '*') {
+                                nick[strlen(nick) - 1] = '\0';
+                                /* isoper */
+                                for(this = mystate->channels; this != NULL; this = this->next) {
+                                        if(isonchan(this, nick)) {
+                                                changeuser(this, nick, 1, -1, -1, -1);
+                                        }
+                                }
+                        }
+			strncpy(userhost, outstring + tpos + 1, USERHOSTLEN);
+                        if(isme(nick)) {
+                                /* this is the forced USERHOST query from 001 */
+                                strncpy(user, userhost, USERHOSTLEN);
+                                ptr = index(user, '@');
+                                strncpy(host, ptr + 1, USERHOSTLEN);
+                                memset(ptr, 0, strlen(ptr));
+                                strncpy(mystate->user, user, USERLEN);
+                                strncpy(mystate->host, host, HOSTLEN);
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(302)->nick %s has host %s\n", nick, outstring + tpos + 1);
+                                oer_debug(OER_DEBUG_INFO, "parseirc(302)->the server says I am %s@%s\n", mystate->user, mystate->host);
 #endif
-			setuserhost(nick, outstring + tpos + 1);
+                                if(index(mystate->flags, (int)'d') != NULL) {
+                                        /* add internal dynamic admin */
+                                        if(addnewadmin(OER_INTERNAL_DYNAMIC_ADMIN, "") == NULL) {
+#ifdef DEBUG
+                                                oer_debug(OER_DEBUG_INFO, "parseirc(302)->failed to add internal dynamic admin %s\n", OER_INTERNAL_DYNAMIC_ADMIN);
+#endif
+                                                return;
+                                        }
+					if(addnewadminmask(OER_INTERNAL_DYNAMIC_ADMIN, userhost) == NULL) {
+#ifdef DEBUG
+                                                oer_debug(OER_DEBUG_INFO, "parseirc(302)->failed to add mask %s to internal dynamic admin %s\n", userhost, OER_INTERNAL_DYNAMIC_ADMIN);
+#endif
+                                                return;
+                                        }
+#ifdef DEBUG
+					oer_debug(OER_DEBUG_INFO, "parseirc(302)->created internal dynamic admin %s with mask %s\n", OER_INTERNAL_DYNAMIC_ADMIN, userhost);
+#endif
+				}
+			} else {
+#ifdef DEBUG
+				oer_debug(OER_DEBUG_INFO, "parseirc(302)->nick %s has userhost %s\n", nick, userhost);
+#endif
+				setuserhost(nick, userhost);
+			}
 		}
 		return;
 	}
@@ -178,7 +211,7 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "315")) {
-                /* end of WHO */
+                /* end of WHO, get bans */
                 restpos = 0;
                 if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
                         return;
@@ -187,6 +220,9 @@ void parseirc(char *serveroutput)
                         return;
                 }
                 this->whoquery = 1;
+		/* make a mode #channel +b query to get all active bans */
+                snprintf(timed_str, WRITE_BUFFER_LENGTH, "MODE %s +b", this->name);
+                timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, timed_str);
                 return;
         }
 	if(!strcasecmp(command, "331")) {
@@ -330,9 +366,36 @@ void parseirc(char *serveroutput)
 		/* make a WHO query to get all userhosts & flags for
 		   nicks on channel */
 		snprintf(timed_str, WRITE_BUFFER_LENGTH, "WHO %s", this->name);
-		timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_PROTECTION, timed_str);
+		timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, timed_str);
 		return;
 	}
+	if(!strcasecmp(command, "367")) {
+		/* channel ban returned from MODE-query */
+                restpos = 0;
+                if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
+                        return;
+                }
+		if((this = getchptr(channel)) == NULL) {
+			return;
+		}
+                /* ban mask */
+                if((restpos = parse(restptr, restpos, " ", userhost, USERHOSTLEN, 0)) < 0) {
+                        return;
+                }
+                if((restpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0)) < 0) {
+                        return;
+                }
+                /* timestamp */
+                if((restpos = parse(restptr, restpos, " ", outstring, STRINGLEN, 0)) < 0) {
+                        return;
+                }
+                addnewchannelban(this, userhost, temp_nick, atol(outstring));
+                return;
+        }
+        if(!strcasecmp(command, "368")) {
+                /* end of channel bans */
+                return;
+        }
 	if(!strcasecmp(command, "372")) {
 		return;
 	}
@@ -529,33 +592,11 @@ void parseirc(char *serveroutput)
 		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
 			return;
 		}
-		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
                 if(isme(nick)) {
-                        /* strip user & host */
-                        strncpy(user, prefix + 1 + tpos, USERHOSTLEN);
-                        ptr = index(user, '@');
-                        memset(ptr, 0, USERHOSTLEN - strlen(user) + 1);
-                        ptr = index(prefix + 1 + tpos, '@');
-                        strncpy(host, ptr + 1, USERHOSTLEN);
-                        /* check if the server agrees with our perception
-                           of our host */
-                        if(strcasecmp(mystate->host, host)) {
-                                strncpy(mystate->host, host, HOSTLEN);
-#ifdef DEBUG
-                                oer_debug(OER_DEBUG_INFO, "parseirc(JOIN)->the server says our host is %s\n", mystate->host);
-#endif
-                        }
-                        /* check if the server agrees with our perception
-                           of our user id */
-                        if(strcasecmp(mystate->user, user)) {
-                                strncpy(mystate->user, user, USERLEN);
-#ifdef DEBUG
-                                oer_debug(OER_DEBUG_INFO, "parseirc(JOIN)->the server says our user id is %s\n", mystate->user);
-#endif
-                        }
                         /* we are not interested in our own joins */
                         return;
                 }
+		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
 		if(userjoined(this, nick, userhost, 0, 0, 0, isfriend(this, nick, userhost)) == NULL) {
 #ifdef DEBUG
 			oer_debug(OER_DEBUG_INFO, "parseirc(JOIN)->userjoined() returned NULL\n");
@@ -860,7 +901,7 @@ void parseirc(char *serveroutput)
                                         sendreply(this->name, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
                                 }
                                 if(!strcasecmp(modeline, "-k")) {
-                                        memset(this->key, 0, CHANLEN);
+                                        memset(this->key, 0, CHANLEN + 1);
                                         snprintf(outstring, STRINGLEN, "deleted channel key from %s", this->name);
                                         sendreply(this->name, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
                                 }
@@ -932,6 +973,12 @@ void parseirc(char *serveroutput)
 #ifdef DEBUG
                                 oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_BAN\n");
 #endif
+				if(strlen(userhost)) {
+                                        snprintf(outstring, STRINGLEN, "%s!%s", temp_nick, userhost);
+                                } else {
+                                        strncpy(outstring, temp_nick, STRINGLEN);
+                                }
+                                addnewchannelban(this, target, outstring, mystate->now);
                                 snprintf(outstring, STRINGLEN, "%s!%s@%s", mystate->nick, mystate->user, mystate->host);
                                 if(wild_match(target, outstring)) {
 #ifdef DEBUG
@@ -956,6 +1003,7 @@ void parseirc(char *serveroutput)
 #ifdef DEBUG
                                 oer_debug(OER_DEBUG_NOISE, "parseirc(MODE)->OER_NTHMODE_UNBAN\n");
 #endif
+                                delchannelban(this, target);
                                 if(this->i_am_op && haschanflags(this, "e") && ispermban(this, target)) {
                                         if(isq(temp_nick) || isservice(temp_nick)) {
                                                 /* allow Q/service to unban */
@@ -1148,12 +1196,11 @@ void parsemsg(char *nick, char *userhost, char *target, char *message)
         ppos = 0;
         is_command = 0;
 	if((ppos = parse(message, ppos, " ", np, NICKLEN, 0)) >= 0) {
-                /* it is possible that parse() will fail if message
-                   has just one word and it is longer than NICKLEN */
+		/* it is possible that parse() will fail if message
+		   has just one word and it is longer than NICKLEN */
                 if(!strcasecmp(np, mystate->nick)) {
                         is_command = 1;
-                }
-                if(!strncasecmp(np, mystate->prefix, strlen(mystate->prefix))) {
+                } else if(!strncasecmp(np, mystate->prefix, strlen(mystate->prefix))) {
                         is_command = 1;
                         ppos = strlen(mystate->prefix);
                 }
@@ -1200,6 +1247,7 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
         time_t idle_per;
         char p1[STRINGLEN + 1];
         char p2[STRINGLEN + 1];
+        char p3[STRINGLEN + 1];
 	char outstring[WRITE_BUFFER_LENGTH + 1];
 	char outstring2[WRITE_BUFFER_LENGTH + 1];
 	char command[STRINGLEN + 1];
@@ -1209,9 +1257,12 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 	char modeline[CHANLEN + 1];
 	char ts[TINYSTRINGLEN + 1];
 	char *restptr;
+	char *ptr;
 	struct channel *this;
 	struct channel *that;
 	struct chanuser *u;
+	struct botuser *user;
+	struct channelban *cb;
 #ifdef DEBUG
         oer_debug(OER_DEBUG_INFO, "parsecommand(\"%s\", \"%s\", \"%s\", \"%s\")\n", target, nick, userhost, commandline);
 #endif
@@ -2355,12 +2406,15 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                 }
                 snprintf(outstring, WRITE_BUFFER_LENGTH, "cloned channel %s to %s", channel, that->name);
                 sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
-		if(!haschanflags(that, "L") && !haschanflags(that, "S")) {
-			return;
+		if(index(mystate->flags, (int)'n') == NULL) {
+			/* old type of last/seen */
+			if(!haschanflags(that, "L") && !haschanflags(that, "S")) {
+				return;
+			}
+			mysqldbname(that->name, outstring2, WRITE_BUFFER_LENGTH);
+			snprintf(outstring, WRITE_BUFFER_LENGTH, "database tables last_%s and seen_%s have to be created manually using the provided oer+MySQL-last.sql and oer+MySQL-seen.sql scripts", outstring2, outstring2);
+			sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
 		}
-		mysqldbname(outstring2, that->name, WRITE_BUFFER_LENGTH);
-		snprintf(outstring, WRITE_BUFFER_LENGTH, "database tables last_%s and seen_%s have to be created manually using the provided oer+MySQL-last.sql and oer+MySQL-seen.sql scripts", outstring2, outstring2);
-		sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
                 return;
 	case OER_WHICHCOMMAND_SSTATS:
 #ifdef DEBUG
@@ -2418,6 +2472,61 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                 snprintf(outstring, WRITE_BUFFER_LENGTH, "my date is %s %s %s", ts, tzname[0], tzname[1]);
                 sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
                 return;
+	case OER_WHICHCOMMAND_USERCOPY:
+#ifdef DEBUG
+                oer_debug(OER_DEBUG_INFO, "parsecommand->USERCOPY\n");
+#endif
+                if((this = getchptr(channel)) == NULL) {
+                        return;
+                }
+                if(!isadmin(this, userhost)) {
+                        return;
+                }
+                ppos = 0;
+                if((ppos = parse(restptr, ppos, " ", p1, STRINGLEN, 0)) < 0) {
+                        return;
+                }
+                if((ppos = parse(restptr, ppos, " ", p2, STRINGLEN, 0)) < 0) {
+                        return;
+                }
+                if((ppos = parse(restptr, ppos, " ", p3, STRINGLEN, 0)) < 0) {
+                        return;
+                }
+                if((that = getchptr(p2)) == NULL) {
+                        return;
+                }
+                ptr = (strlen(p3)) ? p3 : NULL;
+                if((user = usercopy(this, p1, that, ptr)) == NULL) {
+                        return;
+                }
+		if(ptr == NULL) {
+                        snprintf(outstring, WRITE_BUFFER_LENGTH, "%s user %s copied without options to %s", this->name, user->handle, that->name);
+                } else {
+                        snprintf(outstring, WRITE_BUFFER_LENGTH, "%s user %s copied without options to %s as %s", this->name, user->handle, that->name, ptr);
+                }
+                sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
+                return;
+	case OER_WHICHCOMMAND_CLEARBANS:
+#ifdef DEBUG
+                oer_debug(OER_DEBUG_INFO, "parsecommand->CLEARBANS\n");
+#endif
+                if((this = getchptr(channel)) == NULL) {
+                        return;
+                }
+                if(!this->i_am_op) {
+                        return;
+                }
+                if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+                        return;
+                }
+                if(!isatleastopnow(this, nick, userhost)) {
+                        return;
+                }
+                for(cb = this->channelbans; cb != NULL; cb = cb->next) {
+                        mmode_new(this, mystate->now, "-b", cb->ban);
+                        delchannelban(this, cb->ban);
+                }
+                return;
         case OER_WHICHCOMMAND_INVALID:
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->invalid or malformed command %s\n", command);
@@ -2425,7 +2534,3 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 		break;
 	}
 }
-
-
-
-
